//==============================================================================
//
//   SENHA_CR_INFRANEC_FIX
//
//   AUTOR: CLAUDIO LRIOS
//   DATA: 01/03/2014
//   SENHA ELETRONICA USANDO CONTROLE REMOTO INFRAVERMELHO
//   OBJETIVO: LIBERAO DE FECHADURA DE UM RECINTO.
//
//   MICROCONTROLADOR : PIC 12F675  - SENSOR TSOP 1550 (ou similar)
//
//   Obs.Somente para protocolo 'NEC' (LG)
//   Valores de 'command' e 'address' obtidos dos controles
//   para vcr LG 6711R1N156A e de tv LG AKB72915252
//   Para outros controles ter que alterar a tabela conforme respectivos valores
//   Senha atual =3891 (poder ser alterada no programa apenas e recompilando)
//==============================================================================
//
//==============================================================================
//   Obs. Este dispositivo  apenas didtico, sendo montado apenas em protoboard
//   e sendo sujeito a bugs no identificados.
//
//==============================================================================
//Descomente o abaixo SOMENTE se perdeu o byte de calibrao

 //   #define PERDI_BYTE_CALIBRACAO
//==============================================================================


#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(A)  //  sentido das portas feita pelo programador
#ifdef  PERDI_BYTE_CALIBRACAO  // calibrao do oscilador interno do pic
#ROM  0X3FF = {0X34fc}//mxima freq= 0x34fc ;mdia=0x3480; minima=0x3400
#endif
#ZERO_RAM

#ignore_warnings 203,202  //elimina avisos do compilador

//==============================================================================
//   REGISTRADORES
//==============================================================================
#byte gpio   = 0x05
#byte tmr0   = 0x01
#byte tris   = 0x85
#byte t1con   =0x10
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte pir1    =0x0c
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte intcon  =0x8b
#byte status  =0x03
//==============================================================================
//   DEFINIO DE PINOS E SAIDAS
//==============================================================================
#bit  rele  = 0x5.4 //pino 3
#bit  s2    = 0x5.2 //pino 5
#bit  s3    = 0x5.1 //pino 6
#bit  s4    = 0x5.0 //pino 7
#bit  _prog = 0x5.3 //pino 4
#bit  led   = 0x5.5 //pino 2
#bit  _in   = 0x5.3 //pino 4

#bit  cy     = 0x3.0 //carry bit
#bit  on_tmr1 = 0x10.0  //on/off tmr1
#bit  fov1    = 0x0c.0//flag over flow do tmr1
#bit  tmr1if  =  0x0c.0 //flag over flow do tmr1
#bit  tmr0if  = 0x8b.2//flag over tmr0

#define c1  pin_a4
#define c2  pin_a2
#define c3  pin_a1
#define c4  pin_a0
#define prog  pin_a3
#define c5   pin_a5
#define in  pin_a3

//#priority RA,timer0
//==============================================================================
//   CONSTANTES GERAIS
//==============================================================================

int const  qbits = 32;  //quantidade de bits a receber
int const  TLIGADO=30; //tempo mnimo para manter acionamento com boto apertado
int const  TINITPROG=150; // tempo mnimo para iniciar 'modo de programao'
int const  comp_senha=4;  // nmero de digitos da senha
int const  tempo1=255;  // tempo para digitao (low)
int const  tempo2= 255; //tempo para digitao (high)



//==============================================================================
//escolha o tipo de controle remoto que usar

#define  tv  //de tv   (usado AKB72915252 da LG para tv)
//#define  vcr //de video cassete (usado 6711R1N156A da LG para vcr)

//==============================================================================


#ifdef  vcr
int const  _0=0x04;  //obtido de controle 6711r1n156a p/ vcr LG
int const  _1=0x05;
int const  _2=0x06;
int const  _3=0x07;
int const  _4=0x0c;
int const  _5=0x0d;
int const  _6=0x0e;
int const  _7=0x0f;
int const  _8=0x1c;
int const  _9=0x1d;
#endif

#ifdef  tv
int const  _0=0x10;   //obtidos de controle  akb72915252 p/ tv LG
int const  _1=0x11;
int const  _2=0x12;
int const  _3=0x13;
int const  _4=0x14;
int const  _5=0x15;
int const  _6=0x16;
int const  _7=0x17;
int const  _8=0x18;
int const  _9=0x19;
#endif



//==============================================================================
//Defina a senha de 4 digitos aqui.

#rom  0x2100={_3,_8,_9,_1}//altere aqui a sua senha (atual 3891)

// Obrigatrio usar o '_' na frente do nmero para o compilador acertar o valor
// correto do 'command' conforme tabelas acima.
//==============================================================================

//escolha o tempo de acionamento da fechadura

int16 const tempo_acionamento = 10000;  // para 10 segundos

//==============================================================================
// escolha acionamento pulsado ou continuo

//#define continuo   // aciona direto pelo tempo especificado
//#define pulsado    // aciona pulsando pelo tempo especificado
#define retencao     //inverte a situao da sada a cada senha correta inserida
//==============================================================================

//   VARIVEIS GLOBAIS
//==============================================================================

int qb,ax1,cnt1,cnt2,cnt3,cnt4,cnt5,cntp1,T1,T2,dig;
int buffer[4];
int buffer1[comp_senha];
short flag_rok;


//==============================================================================
//   SUBROTINAS E FUNES GERAIS
//==============================================================================



//------------------------------------------------------------------------------
//           COMPARAR SENHA RECEBIDA COM SENHA DA EEPROM
//------------------------------------------------------------------------------
short comparar() {
  int a;
  for  (a=0;a<comp_senha;++a){
        if((buffer1[a]!= read_eeprom(a))){  return (false);}
  }
  return (true);
}
 //==============================================================================
//   INTERRUPO POR MUDANA DE ESTADO - RECEPO DO CONTROLE IR (PADRO 'NEC')
//==============================================================================
#int_RA
void rx_nec_mudanca_gp3() {
 int16 const LIMITE_BYTE_LOW=1200, PULSO_9MS=7000,PULSO_4MS=3800;
 static int16 ref,gt;
 static short ov1,flag_p;


    gpio|=gpio; //ler port para conseguir zera gpif (DS41190C-page 21)
       if(!flag_rok){
           on_tmr1=0; ov1=tmr1if; gt=get_timer1(); set_timer1(0);
           tmr1if=0;  on_tmr1=1;//desliga timer1/pega valor/zera e religa

           cnt1=TLIGADO;
           cnt2=TLIGADO;
           cnt3=TLIGADO;
           cnt4=TLIGADO;
           cnt5=TLIGADO;

           if(!input(in)){         //input='0'
           if((gt>PULSO_9MS)||(ov1)){ qb=qbits;}

             if(flag_p){
              shift_right(&buffer[0],4,(gt>ref));
               if(--qb==0) { qb=qbits;
     if ((buffer[2]+buffer[3]==0xff)){ flag_rok=1;}//extendido apenas
    //   if (!(buffer[0]+buffer[1]+buffer[2]+buffer[3]+0x2)){ flag_rok=1;}//normal
                }
               }
            }
            else{                   //input='1'
                 if((gt>PULSO_4MS)||(ov1)){ qb=qbits; flag_p=0;}
                 else {
                 if((gt<LIMITE_BYTE_LOW)&& (!ov1)&& (!flag_p)){
                    flag_p=1; qb=qbits; ref=gt; ref<<=1; }

                }
            }

         }//if(!flag_rok)
}//void mudanca_gp3()



//==============================================================================

//==============================================================================
//
//                PROGRAMA PRINCIPAL
//
//==============================================================================

void main() {

    //==============================================================================
//   INICIALISAO DE PORTAS E REGISTRADORES
//==============================================================================

   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_timer_1 ( T1_INTERNAL | T1_DIV_BY_1 );
   gpio=0;
   ax1=0;
   tris=0b001000; //tudo sada exceto gp3
   set_timer1(0); //zera tmr1
   tmr1if=0 ; //apaga flag over flow
   on_tmr1=1;//liga timer1
   enable_interrupts(INT_RA3);//habilita interrupo por mudana de estado
   enable_interrupts(global);
   delay_ms(20);
   cntp1=TINITPROG;
   qb=qbits;//carrega quantidade de bits a receber
   flag_rok=0;
   dig=0;
//==============================================================================
//                         LOOP PRINCIPAL
//==============================================================================
   while(true){
   
     delay_ms(10);   //atraso temporizao correta do desligamento (modo Pulso)
//==============================================================================
//==============================================================================
//  CANCELAMENTO DA SENHA TRANSMITIDA APS TEMPO EXCEDIDO
//==============================================================================
    if(!(--t1)){ if(--t2) {
      int a;
     for (a=0; a<comp_senha;++a){ buffer1[a]=0xff; }
     dig=0;
     }
   }

 //=============================================================================
//   ACIONAR SADAS CONFORME PROGRAMAO
//==============================================================================


 if((flag_rok)){


    t1=tempo1; t2=tempo2; //recarga do tempo de apagamento do buffer
    #ifdef tv
    if (buffer[0]==0x04){// compara 'address' se correto
    #endif
    #ifdef vcr
    if (buffer[0]==0x6e){// compara 'address' se correto
    #endif


    buffer1[dig++]=buffer[2];// pega tecla digitada para futura comparao
    led=!led;  //led=1;
    delay_ms(50);//pisca para indicar recepo correta do digito
    led=!led;  //led=0;
    }
    flag_rok=0; //libera nova recepo de tecla
  }

   if(dig==comp_senha){  //se j recebeu toda a senha vai compara-la com eeprom
    int a;
    if (comparar()){
    
    #ifdef continuo
     rele=1; led=1;
     delay_ms(tempo_acionamento);
     rele=0; led=0;
    #endif

    #ifdef pulsado
     for (a=0; a<51; ++a) {
     rele=1; led=1;
     delay_ms(100);
     rele=0;
     
     }

    #endif

    #ifdef  retencao
     rele=!rele;
     led=rele;
     delay_ms(100);
    #endif
     }//se iguais, libera saida
      for (a=0; a<comp_senha;++a){ buffer1[a]=0xff; } //diferentes,apaga
      dig=0;
     #ifdef pulsado
      rele=0; led=0;  //buffer
     #endif
      #ifdef continuo
      rele=0; led=0;  //buffer
     #endif

   }

 }//void while(true)
}//void main



